dnl AC_PROG_CC_FLAG(flag,variable)
AC_DEFUN(AC_PROG_CC_FLAG,
[
flag_pref=$2
if test "x$flag_pref" = x; then
   flag_pref='-'
fi
AC_CACHE_CHECK(whether ${CC-cc} accepts ${flag_pref}$1, ac_cv_prog_cc_$1,
[echo 'void f(){}' > conftest.c
if test -z "`${CC-cc} ${flag_pref}$1 -c conftest.c 2>&1`"; then
  eval ac_cv_prog_cc_$1=yes
else
  eval ac_cv_prog_cc_$1=no
fi
rm -f conftest*
])])


dnl
dnl --with-pamdir
dnl
AC_DEFUN(AC_CHECK_PAM, 
[
    AC_ARG_WITH(pamdir,
    [ --with-pamdir=dir		Where to put pam module [LIBDIR/security]],
    [
        case "$withval" in
        yes|no)
            #
            # Just in case anybody calls it without argument
            #
            AC_MSG_ERROR([--with-pam-dir requires a valid argument])
            ;;
        *)
            PAMDIR="$withval"
            AC_MSG_CHECKING(installing PAM modules in)
            AC_MSG_RESULT(${PAMDIR})
            ;;
        esac
    ],
    [ 
        for dir in /lib/security /usr/lib/security /usr/lib \
               ${prefix}/lib ${prefix}/lib/security
        do
            AC_MSG_CHECKING(if we can install PAM modules in ${dir})
            if test -d ${dir}; then
                AC_MSG_RESULT(yes)
                PAMDIR=${dir}
                break;
	    else 
                AC_MSG_RESULT(no)
            fi
        done
        if test -z "${PAMDIR}"; then
            AC_MSG_ERROR(couldn't figure it out: please use --with-pamdir)
        fi
    ])

    dnl Let's make sure we can find the PAM library :-)
    AC_CHECK_LIB(pam, pam_authenticate)

    dnl Do we use const void *, or just void *?
    AC_MSG_CHECKING(whether PAM prototypes use const pointers)
    AC_EGREP_HEADER([const void \*\*item],security/pam_appl.h,
        [AC_DEFINE(HAVE_PAM_CONST_PROTO) AC_MSG_RESULT(yes)],AC_MSG_RESULT(no))

    dnl Check to see if this is Linux-PAM.  If it doesn't, assume that it
    dnl is Solaris-like.
    dnl Linux-PAM has the <security/pam_misc.h> header and a misc_conv
    dnl conversation function.  We're keying off the misc_conv function
    dnl here.
    AC_CHECK_FUNC(misc_conv,
	[ac_cv_pam_impl="Linux-PAM"; AC_DEFINE(PAM_LINUX_CODEBASE)],
	[ac_cv_pam_impl="Solaris-like"; AC_DEFINE(PAM_SUN_CODEBASE)])
    AC_MSG_CHECKING(PAM implementation)
    AC_MSG_RESULT(${ac_cv_pam_impl})

    AC_SUBST(PAMDIR) 
])dnl

dnl
dnl AC_PROG_KRB5CONFIG(PATH)
dnl
AC_DEFUN(AC_PROG_KRB5CONFIG,
[
    AC_PATH_PROG(krb5config, krb5-config,,$1)dnl
    if test "X$ac_cv_path_krb5config" != "X"; then 
        ac_cv_krb5_prefix=`$ac_cv_path_krb5config --prefix 2>/dev/null`
	ac_cv_krb5_libs=`$ac_cv_path_krb5config --libs 2>/dev/null`
	ac_cv_krb5_cflags=`$ac_cv_path_krb5config --cflags 2>/dev/null`
    fi
])dnl 

dnl AC_PROG_KINIT(PATH)
AC_DEFUN(AC_PROG_KINIT,
[
    AC_PATH_PROG(kinit, kinit,,$1)
])dnl 

dnl
dnl --with-krb5
dnl
AC_DEFUN(AC_CHECK_KRB5, [
    ac_arg_with_krb5=""
    AC_ARG_WITH(krb5,
    [ --with-krb5=dir    Look for Kerberos libs, headers in another directory],
    [
        case "$withval" in
        yes|no)
            #
            # Just in case anybody calls it without argument
            #
            AC_MSG_ERROR([--with-krb5 requires a validargument])
            ;;
        *)
            ac_cv_arg_with_krb5="$withval"
            ;;
        esac
    ])

    dnl Check to see if krb5-config exists.
    AC_PROG_KRB5CONFIG("$ac_cv_arg_with_krb5/bin")

    AC_MSG_CHECKING(where Kerberos is installed)
    if test "$ac_cv_path_krb5config" -a "$ac_cv_krb5_prefix"; then
	AC_MSG_RESULT(found in $ac_cv_krb5_prefix (krb5-config))

    elif test "$ac_cv_arg_with_krb5"; then
	ac_cv_krb5_prefix=$ac_cv_arg_with_krb5
	AC_MSG_RESULT(found in $ac_cv_arg_with_krb5)
    else 
	AC_MSG_ERROR(could not find kerberos)
    fi
    ac_cv_krb5_bin=$ac_cv_krb5_prefix/bin
    ac_cv_krb5_sbin=$ac_cv_krb5_prefix/sbin 
    ac_cv_krb5_lib=$ac_cv_krb5_prefix/lib 
    ac_cv_krb5_rpath=$ac_cv_krb5_prefix/lib 
    ac_cv_krb5_include=$ac_cv_krb5_prefix/include 
    ac_cv_krb5_man=$ac_cv_krb5_prefix/man 
    if test -z "$ac_cv_krb5_libs"; then
        ac_cv_krb5_libs=""
    fi
    if test -z "$ac_cv_krb5_cflags"; then
        ac_cv_krb5_cflags=""
    fi

    dnl
    dnl save state
    dnl
    ac_save__LDFLAGS="$LDFLAGS"
    ac_save__LIBS="$LIBS"
    ac_save__CFLAGS="$CFLAGS"

    if  test -z "$ac_cv_krb5_libs"; then
        LIBS="-lkrb5 -lk5crypto -lcom_err"
        LDFLAGS="-L$ac_cv_krb5_lib"
        CFLAGS="-I$ac_cv_krb5_include"
    else
        LIBS="$ac_cv_krb5_libs"
        LDFLAGS=""
        CFLAGS="$ac_cv_krb5_cflags"
    fi

    AC_MSG_CHECKING(which implementation of Kerberos we have)
    AC_TRY_LINK_FUNC(__heimdal_version,KRB5IMPL="heimdal",KRB5IMPL="mit")
    AC_MSG_RESULT(looks like $KRB5IMPL)
    
    AC_CHECK_LIB(krb5, krb5_init_context)

    dnl If we didn't get the compiler flags from krb5-config, make a
    dnl guess based on the implementation.
    if test -z "$ac_cv_krb5_libs"; then
      if test "$KRB5IMPL" = "mit"; then
        KRB5LIBS="-L${ac_cv_krb5_lib} -lkrb5 -lk5crypto -lcom_err"
        KRB5CFLAGS="-I${ac_cv_krb5_include}"
      elif test "$KRB5IMPL" = "heimdal"; then
        KRB5LIBS="-L${ac_cv_krb5_lib} -lkrb5 -lasn1 -ldes -lroken -lcrypt -lcom_err"
        KRB5CFLAGS="-I${ac_cv_krb5_include}"
      else
        AC_MSG_ERROR(Unknown Kerberos implementation: $KRB5IMPL)
      fi
    else 
        KRB5LIBS="${ac_cv_krb5_libs}"
        KRB5CFLAGS="${ac_cv_krb5_cflags}"
    fi

    ac_save__CPPFLAGS="$CPPFLAGS"
    CPPFLAGS="$KRB5CFLAGS $CPPFLAGS"
    dnl Do we have krb5_free_data_contents, or krb5_xfree? (MIT v. Heimdal)
    AC_EGREP_HEADER(krb5_free_data_contents,krb5.h,
        [COMPAT_SRCS=compat_mit.c COMPAT_OBJS=compat_mit.o],
        [COMPAT_SRCS=compat_heimdal.c COMPAT_OBJS=compat_heimdal.o])

    AC_CHECK_HEADER(krb5.h, :, 
      AC_MSG_ERROR(krb5.h not found!  Please use --with-krb5 to specify an alternate basedir))

    AC_EGREP_HEADER(KRB5KDC_ERR_KEY_EXPIRED,krb5.h,:, 
                AC_DEFINE(HAVE_KRB5KDC_ERR_KEY_EXP))

    dnl Functions that are in the MIT Kerberos 5 API but not Heimdal
    AC_CHECK_FUNCS(krb5_string_to_deltat krb5_free_data_contents \
	krb5_get_prompt_types)
    dnl Functions that are in Heimdal but not MIT Kerberos 5
    AC_CHECK_FUNCS(krb5_cc_copy_cache)

    LDFLAGS="$ac_save__LDFLAGS"
    LIBS="$ac_save__LIBS"
    CFLAGS="$ac_save__CFLAGS"
    CPPFLAGS="$ac_save__CPPFLAGS"

   AC_SUBST(KRB5LIBS)
   AC_SUBST(KRB5IMPL)
   AC_SUBST(KRB5CFLAGS)
   AC_SUBST(COMPAT_SRCS)
   AC_SUBST(COMPAT_OBJS)
])dnl

AC_DEFUN(AC_CHECK_COMPILER, [
dnl What flag will let our compiler build PIC objects?
PICFLAGS=""
LDFLAGS=""

dnl Find out how we create PIC code.
AC_PROG_CC_FLAG(fpic)
if test "$ac_cv_prog_cc_fpic" = yes; then
   PICFLAGS="-fpic";
fi
if test "x$PICFLAGS" = x; then
   AC_PROG_CC_FLAG(KPIC)
   if test "$ac_cv_prog_cc_KPIC" = yes; then
     PICFLAGS="-KPIC";
   fi
fi
if test "x$PICFLAGS" = x; then
   AC_PROG_CC_FLAG(Kpic)
   if test "$ac_cv_prog_cc_Kpic" = yes; then
     PICFLAGS="-Kpic";
   fi
fi

if test "x$PICFLAGS" = x; then
   AC_PROG_CC_FLAG(fno-common)
   if test "$ac_cv_prog_cc_fno_common" = yes; then
     PICFLAGS=-fno-common
     LDCOMBINE='$(CC) -undefined warning -dynamiclib'
   fi
fi

if test "x$PICFLAGS" = x; then
   AC_PROG_CC_FLAG(z,+)
   if test "$ac_cv_prog_cc_z" = yes; then
     PICFLAGS="+z";
   fi
fi

AC_CACHE_CHECK(how to build shared libraries for this platform, ac_cv_prog_ldcombine,
[echo 'void f(){}' > conftest.c
 ${CC-cc} $PICFLAGS -c conftest.c >/dev/null 2>&1
ac_save_LDFLAGS="$LDFLAGS"
 if test -z "`${CC-cc} -shared -o conftest.so conftest.o 2>&1`"; then
    ac_cv_prog_ldcombine='$(CC) -shared'
    if test "x$ac_cv_krb5_rpath" != x; then
	LDFLAGS="$LDFLAGS -Wl,-rpath -Wl,$ac_cv_krb5_rpath"
    fi
 fi
LDFLAGS="$ac_save_LDFLAGS"
 if test "x$ac_cv_prog_ldcombine" = x; then
    if test -z "`${LD-ld} -Bshareable -o conftest.so conftest.o 2>&1`"; then
	ac_cv_prog_ldcombine='$(LD) -Bshareable'
    fi
 fi
 if test "x$ac_cv_prog_ldcombine" = x; then
    if test -z "`${CC-cc} -G -o conftest.so conftest.o 2>&1`"; then
	ac_cv_prog_ldcombine='$(CC) -G'
    fi
 fi
 dnl HPUX check
 if test "x$ac_cv_prog_ldcombine" = x; then
    if test -z "`${LD-ld} -b -o conftest.so conftest.o 2>&1`"; then
	ac_cv_prog_ldcombine='$(LD) -b'
    fi
 fi
 dnl MacOS X check
 if test "x$ac_cv_prog_ldcombine" = x; then
    if test -z "`${CC-cc} -undefined warning -dynamiclib -o conftest.so conftest.o 2>&1`"; then
	ac_cv_prog_ldcombine='$(CC) -undefined warning -dynamiclib'
    fi
 fi
 rm -f conftest*
])

LDCOMBINE=$ac_cv_prog_ldcombine

dnl If we're using gcc, add in the rpath.
if test "$LDCOMBINE" = '$(CC) -shared'; then
  if test "x$ac_cv_krb5_rpath" != x; then
	LDFLAGS="$LDFLAGS -Wl,-rpath -Wl,$ac_cv_krb5_rpath"
  fi
fi

dnl If we're using Solaris CC, add in the rpath.
if test "$LDCOMBINE" = '$(CC) -G'; then
  if test "x$ac_cv_krb5_rpath" != x; then
	LDFLAGS="$LDFLAGS -R$ac_cv_krb5_rpath"
  fi
fi

AC_SUBST(LDCOMBINE)
AC_SUBST(PICFLAGS)

dnl XXX It looks like we only support shared linking.  Linux-PAM
dnl     headers are interested in whether we are shared or not, and
dnl     look at whether the preprocessor symbol `PIC' is defined.
AC_DEFINE(PIC)
])dnl

dnl
dnl AC_INSTALL_OPTIONS
dnl
AC_DEFUN(AC_INSTALL_OPTIONS,
[
    dnl
    dnl --with-man-owner
    dnl
    AC_ARG_WITH(man-owner,
    [ --with-man-owner=user		Which user to install man pages as [default=man]],
        [
            if test -z "$withval"; then
                AC_MSG_WARN([--with-man-owner called without a valid argument])
            else 
                MANOWN="$withval"
                AC_MSG_CHECKING(installing manpages as user)
                AC_MSG_RESULT($MANOWN)
            fi
        ],
        [ 
            MANOWN=man
            AC_MSG_CHECKING(installing manpages as user)
            case "$target_os" in
                solaris2*) MANOWN=bin;;
                *linux*)   MANOWN=root;;
                *) ;;
            esac
            AC_MSG_RESULT($MANOWN)
        ]
    )dnl
    
    AC_SUBST(MANOWN)
    
    dnl
    dnl --with-man-group
    dnl
    AC_ARG_WITH(man-group,
    [ --with-man-group=group		Which group to install man pages as [default=man]],
        [
            if test -z "$withval"; then
                AC_MSG_WARN([--with-man-group called without a valid argument])
            else 
                MANGRP="$withval"
                AC_MSG_CHECKING(installing manpages as group)
                AC_MSG_RESULT($MANGRP)
            fi
        ],
        [ 
            MANGRP=man
            AC_MSG_CHECKING(installing manpages as group)
            case "$target_os" in
                solaris2*) MANGRP=bin;;
                *linux*)   MANGRP=root;;
                *) ;;
            esac
            AC_MSG_RESULT($MANGRP)
        ]
    )dnl
    
    AC_SUBST(MANGRP)
    
    dnl
    dnl --with-bin-owner
    dnl
    AC_ARG_WITH(bin-owner,
    [ --with-bin-owner=user		Which user to install programs as [default=bin]],
        [
            if test -z "$withval"; then
                AC_MSG_WARN([--with-bin-owner called without a valid argument])
            else 
                BINOWN="$withval"
                AC_MSG_CHECKING(installing binary programs as user)
                AC_MSG_RESULT($BINOWN)
            fi
        ],
        [ 
            BINOWN=bin
            AC_MSG_CHECKING(installing binary programs as user)
            case "$target_os" in
                solaris2*) BINOWN=root;;
                freebsd*)  BINOWN=root;;
                *linux*)   BINOWN=root;;
                *) ;;
            esac
            AC_MSG_RESULT($BINOWN)
        ]
    )dnl
    
    AC_SUBST(BINOWN)
    
    dnl
    dnl --with-bin-group
    dnl
    AC_ARG_WITH(bin-group,
    [ --with-bin-group=group		Which group to install programs as [default=bin]],
        [
            if test -z "$withval"; then
                AC_MSG_WARN([--with-bin-group called without a valid argument])
            else 
                BINGRP="$withval"
                AC_MSG_CHECKING(installing binary programs as group)
                AC_MSG_RESULT($BINGRP)
            fi
        ],
        [ 
            BINGRP=bin
            AC_MSG_CHECKING(installing binary programs as group)
            case "$target_os" in
                solaris2*) BINGRP=sys;;
                freebsd*)  BINGRP=sys;;
                *linux*)   BINGRP=root;;
                *) ;;
            esac
            AC_MSG_RESULT($BINGRP)
        ]
    )dnl
    
    AC_SUBST(BINGRP)
    
    dnl
    dnl --with-man-sect
    dnl
    AC_ARG_WITH(man-sect,
    [ --with-man-sect=section	Which section to install manpages as [default=5]],
        [
            if test -z "$withval"; then
                AC_MSG_WARN([--with-man-sect called without a valid argument])
            else 
                MANSEC="$withval"
                AC_MSG_CHECKING(installing manpages in section)
                AC_MSG_RESULT($MANSEC)
            fi
        ],
        [ 
            MANSEC=5
            AC_MSG_CHECKING(installing manpages in section)
            case "$target_os" in
                freebsd*)  MANSEC=8;;
                *) ;;
            esac
            AC_MSG_RESULT($MANSEC)
        ]
    )dnl
    
    AC_SUBST(MANSEC)
])dnl   
