/*
 * pam_krb5.h
 *
 * $Id: pam_krb5.h,v 1.1.1.1.2.4 2001/09/24 20:19:45 nectar Exp $
 */
#ifndef H_PAM_KRB5_H
#define H_PAM_KRB5_H

#include "config.h"
#ifdef PAM_LINUX_CODEBASE
#define PAM_SM_AUTH
#define PAM_SM_ACCOUNT
#define PAM_SM_SESSION
#define PAM_SM_PASSWORD
#endif
#include <security/pam_appl.h>
#include <security/pam_modules.h>

struct passwd;

/* Option processing glue */
enum pam_krb5_option_flags {
#define OPTION(name, flag, val) flag = val,
#include "option_flags.h"
#undef OPTION
};
struct pam_krb5_options {
    unsigned long	 flags;
#define OPTION(name, member) \
    char		*member;
#include "option_strings.h"
#undef OPTION
};

enum pam_krb5_state_flags {
    STATE_WILL_IGNORE =		(1 << 0),
    STATE_SM_AUTHENTICATE =		(1 << 1),
    STATE_PW_EXPIRED =			(1 << 2),
    STATE_AUTH_COMPLETED =		(1 << 3),
};

enum pam_krb5_get_cred_type {
    CRED_TGT = 1,
    CRED_CHANGEPW
};

struct pam_cred;

struct pam_krb5_state {
    unsigned long	 flags;
    struct		 pam_krb5_options opt;
    struct		 pam_cred *c_ucred;
    pam_handle_t	*pamh;
    char		*princ_name;
    const char		*user;
    const char		*service;
    const char		*method;
    krb5_context	 ctx;
    krb5_principal	 princ;
    krb5_ccache		 ccache;
    krb5_creds		*chpw_creds;
};

#define TEST_OPTION(state, flag) ((state)->opt.flags & (flag))
#define SET_OPTION(state, flag)  do {	\
	((state)->opt.flags) |= (flag);	\
} while (0)
#define CLEAR_OPTION(state, flag) do {	\
	((state)->opt.flags) &= ~(flag);	\
} while (0)
#define TEST_STATE(state, flag)  ((state)->flags & (flag))
#define SET_STATE(state, flag)  do {	\
	((state)->flags) |= (flag);		\
} while (0)
#define CLEAR_STATE(state, flag) do {	\
	((state)->flags) &= ~(flag);		\
} while (0)

int	pam_krb5_initialize_method(const char *, pam_handle_t *,
	    struct pam_krb5_state **, int, const char **);
void	pam_krb5_note_result(const char *, struct pam_krb5_state *, int);
int	pam_krb5_become_user(struct pam_krb5_state *, struct passwd *);
int	pam_krb5_unbecome_user(struct pam_krb5_state *);
int	pam_krb5_get_user_info(pam_handle_t *, char *, int, char **);
int	pam_krb5_verify_tgt(struct pam_krb5_state *);
void	pam_krb5_cleanup_ccache(pam_handle_t *, void *, int);
void	pam_krb5_cleanup_creds(pam_handle_t *, void *, int);
void	pam_krb5_debug(const char *format, ...);
void	pam_krb5_log(int priority, const char *format, ...);
int	pam_krb5_setcred(struct pam_krb5_state *, int);
int	pam_krb5_get_cred(struct pam_krb5_state *, char *,
	    enum pam_krb5_get_cred_type , krb5_creds **, int *);
int	pam_krb5_get_authtok(struct pam_krb5_state *,
	    enum pam_krb5_get_cred_type, krb5_creds **);
int	pam_krb5_store_tgt(struct pam_krb5_state *, krb5_creds *);
int	pam_krb5_acct(struct pam_krb5_state *);


krb5_error_code pam_krb5_prompter(krb5_context, void *, const char *,
		    const char *, int, krb5_prompt *);


#define DBG(x) do {						\
	if (state->opt.flags & OPT_DEBUG) pam_krb5_debug##x;	\
} while (0)
#define LOG(x) pam_krb5_log##x

/* MIT / Heimdal compatibility shims and support */
const char	*compat_princ_component(krb5_context, krb5_principal, int);
int		 compat_is_new_password_prompt(krb5_prompt_type *,
		     krb5_prompt *, int);
krb5_error_code	 compat_new_memory_cache(krb5_context, krb5_ccache *);
krb5_error_code	 compat_new_file_cache(krb5_context, krb5_ccache *);

#ifndef HAVE_KRB5_STRING_TO_DELTAT
krb5_error_code	 krb5_string_to_deltat(char *, krb5_deltat *);
#endif
#ifndef HAVE_KRB5_FREE_DATA_CONTENTS
void		 krb5_free_data_contents(krb5_context, krb5_data *);
#endif
#ifndef HAVE_KRB5_GET_PROMPT_TYPES
krb5_prompt_type *krb5_get_prompt_types(krb5_context);
#endif
#ifndef HAVE_KRB5_CC_COPY_CACHE
krb5_error_code	 krb5_cc_copy_cache(krb5_context, krb5_ccache, krb5_ccache);
#endif

#define FREE0(x) do {	\
    if ((x) != NULL) {	\
	free((x));	\
	(x) = NULL;	\
    }			\
} while (0)

#endif /* !H_PAM_KRB5_H */
