/*
 * pam_krb5_acct.c
 *
 * PAM account management functions for pam_krb5
 *
 */

static const char rcsid[] = "$Id: pam_krb5_acct.c,v 1.1.1.1.2.11 2001/09/24 20:19:45 nectar Exp $";

#include <string.h>
#include <syslog.h>	/* syslog */
#include <krb5.h>
#include <com_err.h>
#include "pam_krb5.h"

/* Check authorization of user */
int
pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
    int			 pamret;
    struct		 pam_krb5_state *state;

    pamret = pam_krb5_initialize_method("pam_sm_acct_mgmt", pamh, &state,
	argc, argv);
    if (pamret == PAM_SUCCESS)
    	pamret = pam_krb5_acct(state);
    pam_krb5_note_result("pam_sm_acct_mgmt", state, pamret);
    return pamret;
}

int
pam_krb5_acct(struct pam_krb5_state *state)
{
    int		pamret;

    pamret = PAM_SUCCESS;
    if (!TEST_STATE(state, STATE_AUTH_COMPLETED)) {
    	/* XXX What is the correct response if pam_sm_acct_mgmt is
	 * called without first calling pam_sm_authenticate?
	 */
	DBG(("pam_sm_acct_mgmt: ignore"));
	pamret = PAM_IGNORE;
    }
    if (pamret == PAM_SUCCESS) {
    	if (!krb5_kuserok(state->ctx, state->princ, state->user))
		pamret = PAM_PERM_DENIED;
    	else if (TEST_STATE(state, STATE_PW_EXPIRED))
		pamret = PAM_NEW_AUTHTOK_REQD;
    }

    return pamret;
}
